//////////////////////////////////////////////////////////////////////////////
// ** PORTSCAN - Win32 application to obtain host info and scan IP ports **
// THIS SOFTWARE IS Copyright 1998 by Bob Frazier - All rights reserved
// It has been provided to the public as "copy-left" according to the terms
// of the GNU GENERAL PUBLIC LICENSE at http://www.gnu.org/copyleft/gpl.html
//
// THIS CODE IS PROVIDED 'AS-IS' with NO WARRANTEE WHATSOEVER
// USE IT AT YOUR OWN RISK - Bob Frazier CLAIMS NO RESPONSIBILITY FOR
// ANY DAMAGES, CONSEQUENTIAL OR OTHERWISE, THAT MAY RESULT FROM
// USE OR ABUSE OF THIS PROGRAM, WHETHER IN ITS ORIGINAL OR MODIFIED STATE.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// For a copy of the GNU General Public License, you can visit the
// 'gnu.org' web site at http://www.gnu.org/, or write to the
// Free Software Foundation, Inc.
// 59 Temple Place - Suite 330
// Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////
// InetAddress.cpp : implementation file
//

#include "stdafx.h"
#include "portscan.h"
#include "InetAddress.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CInetAddress

IMPLEMENT_DYNAMIC(CInetAddress,CObject);

CInetAddress::CInetAddress(LPCSTR szAddr, UINT uiDefPort /* = 0 */)
{
  m_csAddr = szAddr;

  m_csAddr.TrimRight();
  m_csAddr.TrimLeft();

  sin_family = AF_INET;  // required
  sin_port = 0;                            // pre-load with zero
  memset(&sin_addr, 0, sizeof(sin_addr));  // pre-load with zeros, first
  memset(&sin_zero, 0, sizeof(sin_zero));  // zero out unused portion

  m_bIsValid = FALSE;

  int i1;

  i1 = m_csAddr.Find(':');  // the only ':' should be for a port

  CString csAddr = m_csAddr;

  if(i1 >= 0)
  {
    sin_port = atoi(csAddr.Mid(i1 + 1));

    if(!sin_port)
      sin_port = uiDefPort;

    csAddr = csAddr.Left(i1);
    csAddr.TrimRight();
  }
  else
  {
    sin_port = uiDefPort;
  }

  sin_port = htons(sin_port);  // ensure byte order is reversed

  if(!csAddr)
  {
    // assume current machine - get machine's default host name

    gethostname(csAddr.GetBufferSetLength(MAX_PATH), MAX_PATH);
    csAddr.ReleaseBuffer(-1);

    m_csAddr.Format("%s:%d", (LPCSTR)csAddr, (UINT)htons(sin_port));
  }


  HOSTENT *pH = NULL;

  // is address a #.#.#.# or a name?

  BOOL bIsAddress = IsAddress(csAddr); // alphanumeric?

  if(!bIsAddress)  
  {
    pH = gethostbyname(csAddr);
  }
  else
  {
    char cAddr[4]={0,0,0,0};

    if(GetAddress(csAddr, cAddr))
    {
      sin_addr.S_un.S_un_b.s_b1 = cAddr[0];
      sin_addr.S_un.S_un_b.s_b2 = cAddr[1];
      sin_addr.S_un.S_un_b.s_b3 = cAddr[2];
      sin_addr.S_un.S_un_b.s_b4 = cAddr[3];

      pH = gethostbyaddr(cAddr, sizeof(cAddr), PF_INET);

      if(!pH)  // special case - specified an address...
      {
        if(::MessageBox(NULL, csAddr + " - address does not resolve - use it anyway?",
                        "* PORTSCAN - WARNING *",
                        MB_SETFOREGROUND | MB_YESNO | MB_ICONASTERISK)
           == IDYES)
        {
          m_bIsValid = TRUE;  // if I get here, it worked ok
          m_csAddr.Format("%s:%d", (LPCSTR)csAddr, htons(sin_port));  // just keep THIS
        }

        return;
      }
    }
  }

  if(!pH)
  {
    return;  // no further processing (error)
  }

  if(pH->h_name)  // keep copy of name/port I connected to
  {
    m_csAddr.Format("%s:%d", (LPCSTR)pH->h_name, (UINT)htons(sin_port));
  }

  // if there's an alias, put the first one in m_csAlias

  if(pH->h_aliases && pH->h_aliases[0])
    m_csAlias = pH->h_aliases[0];

  ASSERT(pH->h_addrtype == PF_INET);
  ASSERT(pH->h_length == 4);

  if(!pH->h_addr_list || !pH->h_addr_list[0])
  {
    return;  // no further processing (error)
  }

  if(bIsAddress)
  {
    SOCKADDR_IN sa;
    memset(&sa, 0, sizeof(sa));

    sa.sin_family = sin_family;
    sa.sin_port = sin_port;

    if(pH->h_length >= 1)
      sa.sin_addr.S_un.S_un_b.s_b1 = pH->h_addr_list[i1][0];
    if(pH->h_length >= 2)
      sa.sin_addr.S_un.S_un_b.s_b2 = pH->h_addr_list[i1][1];
    if(pH->h_length >= 3)
      sa.sin_addr.S_un.S_un_b.s_b3 = pH->h_addr_list[i1][2];
    if(pH->h_length >= 4)
      sa.sin_addr.S_un.S_un_b.s_b4 = pH->h_addr_list[i1][3];

    if(sa.sin_addr.S_un.S_addr != sin_addr.S_un.S_addr)  // do they match?
      m_aAlternates.Add(sa);  // if NOT, make this one the 1st alternate
  }
  else
  {
    if(pH->h_length >= 1)
      sin_addr.S_un.S_un_b.s_b1 = pH->h_addr_list[0][0];
    if(pH->h_length >= 2)
      sin_addr.S_un.S_un_b.s_b2 = pH->h_addr_list[0][1];
    if(pH->h_length >= 3)
      sin_addr.S_un.S_un_b.s_b3 = pH->h_addr_list[0][2];
    if(pH->h_length >= 4)
      sin_addr.S_un.S_un_b.s_b4 = pH->h_addr_list[0][3];
  }

  // now, as long as there are alternates, add them to the 'm_aAlternates' array

  for(i1=1; pH->h_addr_list[i1]; i1++)
  {
    SOCKADDR_IN sa;
    memset(&sa, 0, sizeof(sa));

    sa.sin_family = sin_family;
    sa.sin_port = sin_port;

    if(pH->h_length >= 1)
      sa.sin_addr.S_un.S_un_b.s_b1 = pH->h_addr_list[i1][0];
    if(pH->h_length >= 2)
      sa.sin_addr.S_un.S_un_b.s_b2 = pH->h_addr_list[i1][1];
    if(pH->h_length >= 3)
      sa.sin_addr.S_un.S_un_b.s_b3 = pH->h_addr_list[i1][2];
    if(pH->h_length >= 4)
      sa.sin_addr.S_un.S_un_b.s_b4 = pH->h_addr_list[i1][3];

    if(sa.sin_addr.S_un.S_addr != sin_addr.S_un.S_addr)  // do they match?
      m_aAlternates.Add(sa);  // it not, go ahead and add it (make sure I don't duplicate)
  }

  m_bIsValid = TRUE;  // if I get here, it worked ok
}

CInetAddress::~CInetAddress()
{
}

SOCKADDR_IN CInetAddress::operator [] (int iIndex)
{
  if(!iIndex)
  {
    return((SOCKADDR_IN)*this);
  }

  if(iIndex < 0 || iIndex > m_aAlternates.GetSize())
  {
    SOCKADDR_IN sa;
    memset(&sa, 0, sizeof(sa));

    return(sa);
  }

  return(m_aAlternates[iIndex - 1]);
}


// static member "helper" utility functions

BOOL CInetAddress::IsAddress(LPCSTR szAddr)  // TRUE if #.#.#.#; false otherwise
{
  // is address a #.#.#.# or a name?

  CString csAddr = szAddr;

  int i1;
  for(i1=0; i1 < csAddr.GetLength(); i1++)
  {
    if((csAddr[i1] < '0' || csAddr[i1] > '9') &&
       csAddr[i1] != '.')
      break;
  }

  return(i1 >= csAddr.GetLength());
}

BOOL CInetAddress::GetAddress(LPCSTR szAddr, char cAddr[4])
{
  LPCSTR lp1 = szAddr;
  unsigned long aL[4];

  cAddr[0] = cAddr[1] = cAddr[2] = cAddr[3] = 0;
  aL[0] = aL[1] = aL[2] = aL[3] = 0;

  int i1;
  for(i1=0; i1 < 4; i1++)
  {
    while(*lp1 && *lp1 != '.')
    {
      while(*lp1 <= ' ')
        lp1++;
      
      if(*lp1 < '0' || *lp1 > '9')  // not likely
        break;
      
      aL[i1] *= 10L;
      aL[i1] += (BYTE)(*lp1 - '0');
      
      lp1++;
    }

    if(!*lp1 || *lp1 != '.')
      break;
    
    lp1++;  // go past the '.'
  }

  if(!*lp1)  // this means 'good result'
  {
    // assume any remaining items are zeros and assign them as such,
    // unless the address was entered as a non-standard long integer
    // or "less than 3 dot" address.

    if(i1 < 1)
    {
      aL[1] = aL[0] & 0xffffff;
      aL[0] = (aL[0] >> 24) & 0xff;
    }
    if(i1 < 2)
    {
      aL[2] = aL[1] & 0xffff;
      aL[1] = (aL[1] >> 16) & 0xff;
    }
    if(i1 < 3)
    {
      aL[3] = aL[2] & 0xff;
      aL[2] = (aL[2] >> 8) & 0xff;
    }
       
    for(i1=0; i1 < 4; i1++)
    {
      cAddr[i1] = (unsigned char)(aL[i1] & 0xff);
    }
  }

  return(i1 >= 4);
}

BOOL CInetAddress::IsAddressRange(LPCSTR szAddr)  // TRUE if #.#.#.# with '*'
{
  // is address a #.#.#.# or a name?

  CString csAddr = szAddr;
  BOOL bAssToRisk = FALSE;

  int i1;
  for(i1=0; i1 < csAddr.GetLength(); i1++)
  {
    if((csAddr[i1] < '0' || csAddr[i1] > '9') &&
       csAddr[i1] != '.')
    {
      if(csAddr[i1] == '*' &&    // if it's an asterisk...
         (i1 == 0 || csAddr[i1 - 1] == '.') &&  // previous and next are/were '.'
         ((i1 + 1) >= csAddr.GetLength() || csAddr[i1 + 1] == '.'))
      {
        bAssToRisk = TRUE;
      }
      else
      {
        break;
      }
    }
  }

  return(bAssToRisk && i1 >= csAddr.GetLength());
}

BOOL CInetAddress::GetAddressRange(LPCSTR szAddr, int cAddr[4])
{
  LPCSTR lp1 = szAddr;

  cAddr[0] = cAddr[1] = cAddr[2] = cAddr[3] = 0;

  int i1;
  for(i1=0; i1 < 4; i1++)
  {
    while(*lp1 && *lp1 != '.')
    {
      while(*lp1 <= ' ')
        lp1++;

      if(*lp1 == '*')
      {
        cAddr[i1] = -1;
        lp1++;

        break;
      }
      else if(*lp1 < '0' || *lp1 > '9')  // not likely
      {
        break;
      }

      cAddr[i1] *= 10;
      cAddr[i1] += (BYTE)(*lp1 - '0');
      
      lp1++;
    }
    
    if(*lp1 && *lp1 != '.')
    {
      break;  // not likely
    }
    else if(!*lp1)
    {
      // assume remaining items are zeros and assign them as such
      
      while(i1 < 3)
        cAddr[++i1] = 0;
    }
    else
    {
      lp1++;  // go past the '.'
    }
  }

  return(i1 >= 4);
}
