//////////////////////////////////////////////////////////////////////////////
// ** PORTSCAN - Win32 application to obtain host info and scan IP ports **
// THIS SOFTWARE IS Copyright 1998 by Bob Frazier - All rights reserved
// It has been provided to the public as "copy-left" according to the terms
// of the GNU GENERAL PUBLIC LICENSE at http://www.gnu.org/copyleft/gpl.html
//
// THIS CODE IS PROVIDED 'AS-IS' with NO WARRANTEE WHATSOEVER
// USE IT AT YOUR OWN RISK - Bob Frazier CLAIMS NO RESPONSIBILITY FOR
// ANY DAMAGES, CONSEQUENTIAL OR OTHERWISE, THAT MAY RESULT FROM
// USE OR ABUSE OF THIS PROGRAM, WHETHER IN ITS ORIGINAL OR MODIFIED STATE.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// For a copy of the GNU General Public License, you can visit the
// 'gnu.org' web site at http://www.gnu.org/, or write to the
// Free Software Foundation, Inc.
// 59 Temple Place - Suite 330
// Boston, MA  02111-1307, USA.
//////////////////////////////////////////////////////////////////////////////
// portscanDlg.cpp : implementation file
//

#include "stdafx.h"
#include "portscan.h"
#include "portscanDlg.h"
#include "InetAddress.h"
#include "SockThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#define THREAD_COUNT 128 /* initially, 128 threads - end-users can change this */
#define THREAD_COUNT (1L << m_iThreadCount)
#define WM_THREADEXIT (WM_USER + 123)

/////////////////////////////////////////////////////////////////////////////
// CPortscanDlg dialog


CMutex CPortscanDlg::m_Mutex(FALSE, "PORTSCANDLG_MUTEX");  // for synchronization
CArray<CWinThread *,CWinThread *> CPortscanDlg::m_aThreads;  // current array of threads

CPortscanDlg::CPortscanDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPortscanDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPortscanDlg)
	m_csIPPort = _T("");
	m_iType = -1;
	m_csStatus = _T("");
	m_iThreadCount = -1;
	//}}AFX_DATA_INIT

	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

  m_iType = 0;
  m_iThreadCount = 5;  // default is 32 threads
}

void CPortscanDlg::AddThread(CWinThread *pThread)
{
CSingleLock lock(&m_Mutex, TRUE);

  int i1;
  for(i1=0; i1 < m_aThreads.GetSize(); i1++)
  {
    if(m_aThreads[i1] == pThread)
    {
      break;
    }
  }

  if(i1 >= m_aThreads.GetSize())
    m_aThreads.Add(pThread);
}

void CPortscanDlg::EndThread(CWinThread *pThread)
{
CSingleLock lock(&m_Mutex, TRUE);

  int i1;
  for(i1=0; i1 < m_aThreads.GetSize(); i1++)
  {
    if(m_aThreads[i1] == pThread)
    {
      m_aThreads.RemoveAt(i1);
      break;
    }
  }

  lock.Unlock();

  while(!theApp.m_pMainWnd->PostMessage(WM_THREADEXIT, 0, 0))  // this is a wakeup call
    Sleep(0);  // wait 'till I can post this successfully

  TRACE("Thread ended - %08xH\r\n", pThread->m_nThreadID);
}

void CPortscanDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPortscanDlg)
	DDX_Control(pDX, IDC_THREAD_COUNT, m_wndThreadCount);
	DDX_Control(pDX, IDC_EXIT, m_btnExit);
	DDX_Control(pDX, IDOK, m_btnScan);
	DDX_Control(pDX, IDHELP, m_btnHelp);
	DDX_Control(pDX, IDCANCEL, m_btnCancel);
	DDX_Control(pDX, IDC_SAVE, m_btnSave);
	DDX_Control(pDX, IDC_LIST, m_wndList);
	DDX_Text(pDX, IDC_IP_PORT, m_csIPPort);
	DDX_Radio(pDX, IDC_PORT, m_iType);
	DDX_Text(pDX, IDC_STATUS, m_csStatus);
	DDX_CBIndex(pDX, IDC_THREAD_COUNT, m_iThreadCount);
	//}}AFX_DATA_MAP

  CSingleLock lock(&m_Mutex, TRUE);

  if(!m_aThreads.GetSize())
  {
    m_btnCancel.EnableWindow(0);

    m_btnHelp.EnableWindow(1);
    m_btnSave.EnableWindow(1);
    m_btnScan.EnableWindow(1);

    m_btnExit.EnableWindow(1);  // force it
  }
  else
  {
    m_btnHelp.EnableWindow(0);
    m_btnSave.EnableWindow(0);
    m_btnScan.EnableWindow(0);
  }
}

BEGIN_MESSAGE_MAP(CPortscanDlg, CDialog)
	//{{AFX_MSG_MAP(CPortscanDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDHELP, OnHelp)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	ON_WM_SETCURSOR()
	//}}AFX_MSG_MAP

  ON_MESSAGE(WM_THREADEXIT, OnThreadExit)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPortscanDlg message handlers

BOOL CPortscanDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
  m_Font.CreateFont(HIWORD(GetDialogBaseUnits()), 0, 0, 0, FW_NORMAL, 0, 0, 0,
                    ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_TT_ALWAYS,
                    DEFAULT_QUALITY, FIXED_PITCH | FF_DONTCARE, "Courier New");

  m_wndList.SetFont(&m_Font);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

LRESULT CPortscanDlg::OnThreadExit(WPARAM,LPARAM)
{
  UpdateData(0);  // this should fix button enable/disable

  return(0);
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPortscanDlg::OnPaint() 
{
	if(IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

BOOL CPortscanDlg::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
  CSingleLock lock(&m_Mutex, TRUE);

  if(!m_aThreads.GetSize() || nHitTest != HTCLIENT)
  {
    lock.Unlock();
    
    // set to default when no threads running or not in client area

  	return CDialog::OnSetCursor(pWnd, nHitTest, message);
  }
  else
  {
    lock.Unlock();
    
    SetCursor(::LoadCursor(NULL, IDC_WAIT));  // set to hourglass

    return(TRUE);
  }
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPortscanDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CPortscanDlg::OnHelp() 
{
  AfxMessageBox("This application scans for IP addresses listening on a particular port, "
                "or for listening ports on a particular IP address.  It uses multiple "
                "threads of execution to minimize the time delay.  You should have a fast "
                "internet connection to maximize its effectiveness in the scan.\n"
                "To scan for listening IP addresses, select 'Scan IP' and enter the "
                "desired IP mask and port number in the 'IP/Port' box, as a 4-number IP "
                "address followed by a ':' and the port number as a decimal value "
                "(such as 80 for www service).  Each number value in the IP mask must be "
                "a value between 0 and 255, or a single asterisk '*'.  An asterisk will "
                "cause the numbers in that section to scan for ports from 0 to 255.  You "
                "can specify multiple asterisks in the address, such as '192.168.*.*'.  "
                "A valid entry might be something like '192.168.*.*:80'.\n"
                "For information on ports, see RFC1700.  Commonly used ports are 7 (echo),"
                "21 (ftp), 23 (telnet), 25 (smtp), 79 (finger), 80 (http), 110 (pop3), and 139 (netbios).\n"
                "To scan for listening ports on a single IP, select 'Scan Ports' and "
                "enter the desired IP address (server address) as a 4-number IP address "
                "(such as 192.168.0.1) or known server name (such as www.microsoft.com).\n\n"
                "PLEASE do not ABUSE this application, as it can adversely affect bandwidth.",
                MB_ICONASTERISK);
}

void CPortscanDlg::OnSave() 
{
  CFileDialog dlg(FALSE);

  if(dlg.DoModal() == IDOK)
  {
    HANDLE hFile = CreateFile(dlg.GetPathName(),GENERIC_READ | GENERIC_WRITE,
                              0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL,
                              NULL);

    if(hFile == INVALID_HANDLE_VALUE)
    {
      AfxMessageBox("Unable to open desired file");
      return;
    }

    CString csTemp, csBuf;

    int i1;
    for(i1=0; i1 < m_wndList.GetCount(); i1++)
    {
      m_wndList.GetText(i1, csTemp);

      csBuf += csTemp + "\r\n";
    }

    DWORD cb1;
    if(!WriteFile(hFile, (LPCSTR)csBuf, csBuf.GetLength(), &cb1, NULL) ||
       cb1 != (DWORD)csBuf.GetLength())
    {
      AfxMessageBox("Write error on output file");
    }

    CloseHandle(hFile);
  }
}

void CPortscanDlg::OnOK() 
{
//	CDialog::OnOK();

  m_wndList.ResetContent();
  UpdateData(1);

  m_btnCancel.EnableWindow(1);  // I do this MANUALLY before creating threads

  m_btnHelp.EnableWindow(0);  // disable THESE manually as well
  m_btnSave.EnableWindow(0);
  m_btnScan.EnableWindow(0);

  if(m_iType == 0)
  {
    // scan ports for a given IP address
    // NOTE:  if user specifies named address, do a scan for
    //        every IP address.  This may requires multiple
    //        sets of threads, but that's not a problem.

    CInetAddress addr(m_csIPPort,0);  // should be an address with no port

    if(!addr.IsValid())
    {
      AfxMessageBox("Internet address is not valid");

      UpdateData(0);
      return;
    }

    CString csTemp;
    csTemp.Format("         SCANNING:  %03d.%03d.%03d.%03d",
                  (BYTE)addr[0].sin_addr.S_un.S_un_b.s_b1,
                  (BYTE)addr[0].sin_addr.S_un.S_un_b.s_b2,
                  (BYTE)addr[0].sin_addr.S_un.S_un_b.s_b3,
                  (BYTE)addr[0].sin_addr.S_un.S_un_b.s_b4);

    m_wndList.AddString(csTemp);

    int i1;
    if(addr.GetDimensions() > 1)
    {
      AfxMessageBox("This address has multiple IP addresses.  Only the first will be scanned.");

      for(i1=1; i1 < addr.GetDimensions(); i1++)
      {
        csTemp.Format("        (alt IP)    %03d.%03d.%03d.%03d",
                      (BYTE)addr[i1].sin_addr.S_un.S_un_b.s_b1,
                      (BYTE)addr[i1].sin_addr.S_un.S_un_b.s_b2,
                      (BYTE)addr[i1].sin_addr.S_un.S_un_b.s_b3,
                      (BYTE)addr[i1].sin_addr.S_un.S_un_b.s_b4);

        m_wndList.AddString(csTemp);
      }
    }

    // spawn 'THREAD_COUNT' threads

    CWaitCursor wait;

    for(i1=0; i1 < THREAD_COUNT; i1++)
    {
      csTemp.Format("%d", i1);

      SetDlgItemText(IDC_STATUS, csTemp);

      CSockThread *pT = (CSockThread *)AfxBeginThread(RUNTIME_CLASS(CSockThread),
                                                      THREAD_PRIORITY_NORMAL - 1,
                                                      0, CREATE_SUSPENDED);
      if(!pT)
      {
        AfxMessageBox("ERROR:  unable to create enough threads to complete task.\n"
                      "Select a smaller number of threads before trying again.");

        OnCancel();

        return;
      }

      pT->m_aIP[0] = addr.sin_addr.S_un.S_un_b.s_b1;
      pT->m_aIP[1] = addr.sin_addr.S_un.S_un_b.s_b2;
      pT->m_aIP[2] = addr.sin_addr.S_un.S_un_b.s_b3;
      pT->m_aIP[3] = addr.sin_addr.S_un.S_un_b.s_b4;

      pT->m_iIncr = THREAD_COUNT;
      pT->m_iMask = i1;
      pT->m_iPort = -1;  // scan all of them

      pT->ResumeThread();
    }
  }
  else
  {
    // check for a range of addresses

    CString csIP = m_csIPPort;
    int iPort = 7;  // default is echo port

    int i1 = csIP.Find(':');  // get the ':' for the port #

    if(i1 >= 0)
    {
      iPort = atoi(csIP.Mid(i1 + 1));

      csIP = csIP.Left(i1);
      csIP.TrimRight();
    }

    int aIP[4];

    if(!csIP.GetLength() ||
       (!CInetAddress::IsAddress(csIP) && !CInetAddress::IsAddressRange(csIP)) ||
       !CInetAddress::GetAddressRange(csIP, aIP))
    {
      AfxMessageBox("You must enter a valid IP address, substituting '*' for "
                    "those portions that you want to scan, and specify an optional "
                    "port number, in a format as follows:\n"
                    "192.168.0.*:80\n"
                    "The above IP address and port will scan all addresses between "
                    "192.168.0.0 and 192.168.0.255 for port 80.");

      UpdateData(0);
      return;
    }

    int iThreadCount = THREAD_COUNT;
    if(iThreadCount > 256)
    {
      // can't have more than 256

      AfxMessageBox("WARNING: Algorithm limits this scan to 256 threads maximum",
                    MB_ICONASTERISK);

      iThreadCount = 256;
    }

    // spawn 'THREAD_COUNT' threads

    CWaitCursor wait;

    CString csTemp;

    csTemp.Format("         SCANNING: %s for port %d",
                  (LPCSTR)csIP, iPort);

    m_wndList.AddString(csTemp);

    for(i1=0; i1 < iThreadCount; i1++)
    {
      csTemp.Format("%d", i1);

      SetDlgItemText(IDC_STATUS, csTemp);

      CSockThread *pT = (CSockThread *)AfxBeginThread(RUNTIME_CLASS(CSockThread),
                                                      THREAD_PRIORITY_NORMAL - 1,
                                                      0, CREATE_SUSPENDED);
      if(!pT)
      {
        AfxMessageBox("ERROR:  unable to create enough threads to complete task.\n"
                      "Select a smaller number of threads before trying again.");

        OnCancel();

        return;
      }

      pT->m_aIP[0] = aIP[0];
      pT->m_aIP[1] = aIP[1];
      pT->m_aIP[2] = aIP[2];
      pT->m_aIP[3] = aIP[3];

      pT->m_iIncr = iThreadCount;
      pT->m_iMask = i1;
      pT->m_iPort = iPort;  // scan only 1

      pT->ResumeThread();
    }
  }


  UpdateData(0);
}

void CPortscanDlg::OnCancel() 
{
CSingleLock lock(&m_Mutex, TRUE);

  int i1;
  for(i1=0; i1 < m_aThreads.GetSize(); i1++)
  {
    if(m_aThreads[i1])
    {
      m_aThreads[i1]->PostThreadMessage(WM_QUIT, 0, 0);
    }
  }

  lock.Unlock();

  m_btnCancel.EnableWindow(0);
}

void CPortscanDlg::OnExit() 
{
	CDialog::OnCancel();
}


